unit SoundMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Buttons, ToolWin, ImgList, Danhint, ComboBox97,
  mmsystem, CyDSounderTr, ExtCtrls, slider, CyDRecorder, registry, msacm,
  Gauges;

type
  TSounderForm = class(TForm)
    FreqBar: TTrackBar;
    ToneBar: TTrackBar;
    VolumeLBar: TTrackBar;
    VolumeRBar: TTrackBar;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    ToolBar1: TToolBar;
    OpenButton: TToolButton;
    ImageList1: TImageList;
    PlayButton: TToolButton;
    StopButton: TToolButton;
    ToolButton4: TToolButton;
    PouseButton: TToolButton;
    DanHint1: TDanHint;
    OpenDialog1: TOpenDialog;
    SoundTimer: TTimer;
    RecButton: TToolButton;
    Label4: TLabel;
    Label5: TLabel;
    ToolButton1: TToolButton;
    Bevel1: TBevel;
    ToolButton2: TToolButton;
    OptionButton: TToolButton;
    ExitButton: TToolButton;
    ConvButton: TToolButton;
    ToolButton3: TToolButton;
    ProgressBar1: TProgressBar;
    NewButton: TToolButton;
    SaveDialog1: TSaveDialog;
    Label6: TLabel;
    Label7: TLabel;
    FreqBox: TEdit;
    BitsBox: TEdit;
    ChanelsBox: TEdit;
    PositionBar: TTrackBar;
    procedure FormShow(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure PlayButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure PouseButtonClick(Sender: TObject);
    procedure SoundTimerTimer(Sender: TObject);
    procedure FreqBarChange(Sender: TObject);
    procedure ToneBarChange(Sender: TObject);
    procedure VolumeLBarChange(Sender: TObject);
    procedure PositionBarMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RecButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure OptionButtonClick(Sender: TObject);
    procedure ExitButtonClick(Sender: TObject);
    procedure PositionBarMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ConvButtonClick(Sender: TObject);
    procedure NewButtonClick(Sender: TObject);
  private
    { Private declarations }
    RestartPlay:Boolean;
    procedure InitFile(hFile: THandle; wfx1:PWAVEFORMATEX);
    procedure UpdateWavFile(hFile: THandle; wfx1:PWAVEFORMATEX; newDataSize:Integer);
    procedure FillDevice;
    procedure OpenWaveFile;
    procedure EnableBut(Stat:Boolean);
    procedure InitSoundControls;
    procedure ReadOption;
    procedure WriteOption;
    procedure WavSaveToFile;
    procedure CalculateRestart;
    procedure ConvertToFormat(oldFormat,newFormat:PWAVEFORMATEX);
    function InitNewFile(hFile:THandle):Integer;
  public
    { Public declarations }
    CyDSounder1:TCyDSounder;
    SoundRecorder1:TSoundRecorder;
    WavFileName:String;
    wFileSize,DataSize,DataOffset:Int64;
    w_hData:THandle;
    wfx:TWAVEFORMATEX;
    ACMwfx:TWAVEFORMATEX;
    w_Pause, w_Restart, w_Stop:Boolean;
    w_PlayPosition,w_RestartPosition:Int64;
    w_DeviceID:Integer;
    w_hPlay:HWAVEOUT;
    TempWavFile:String;
    procedure SaveRecFile;
  end;

var
  SounderForm: TSounderForm;

implementation

uses SoundOptionDlg, NewFile;

{$R *.DFM}

procedure WaveInCheck(res:MMResult);
var
 ErrText:array [0..MAXERRORLENGTH+1] of char;
begin
 if res=MMSYSERR_NOERROR then exit;
 waveInGetErrorText(res,ErrText,Sizeof(ErrText));
 MessageBox(SounderForm.Handle,ErrText, 'Error', MB_OK+MB_ICONEXCLAMATION);
end;

procedure WaveOutCheck(res:MMResult);
var
 Errtext:array[0..MAXERRORLENGTH+1] of char;
begin
 if Res=MMSYSERR_NOERROR then exit;
 waveOutGetErrorText(res,ErrText,SizeOf(ErrText));
 MessageBox(SounderForm.Handle,ErrText,'Error',MB_OK+MB_ICONINFORMATION);
end;

procedure TSounderForm.FillDevice;
var
 Num,i:Integer;
 WaveCaps:TWaveOutCaps;
 pwoc:TWaveOutCaps;
begin
 SoundOptionForm.AudioDeviceCBox.Items.Clear;
 Num:=waveOutGetNumDevs;
 for i:=0 to Num-1 do
  begin
   WaveOutGetDevCaps(i,@WaveCaps,sizeof(WaveCaps));
   SoundOptionForm.AudioDeviceCBox.Items.Add(WaveCaps.szPName);
  end;
 SoundOptionForm.AudioDeviceCBox.Items.Add('Automatic select audio device');

 SoundOptionForm.RecDeviceCBox.Items.Clear;
 Num:=waveInGetNumDevs;
 for i:=0 to Num-1 do
  begin
   WaveInGetDevCaps(i,@WaveCaps,sizeof(WaveCaps));
   SoundOptionForm.RecDeviceCBox.Items.Add(WaveCaps.szPName);
  end;
 SoundOptionForm.RecDeviceCBox.Items.Add('Automatic select record device');

 waveOutGetDevCaps(w_DeviceID, @pwoc, sizeof(pwoc));
 if (pwoc.dwSupport and WAVECAPS_PITCH)>0 then
  FreqBar.Enabled:=true;
 if (pwoc.dwSupport and WAVECAPS_PLAYBACKRATE)>0 then
  ToneBar.Enabled:=true;
 if (pwoc.dwSupport and WAVECAPS_VOLUME)>0 then
  begin
   VolumeLBar.Enabled:=true;
   VolumeRBar.Enabled:=true;
  end;
end;

procedure TSounderForm.FormShow(Sender: TObject);
begin
 EnableBut(false);
 DanHint1.HintActive:=true;
 FillDevice;
 ReadOption;
 RestartPlay:=false;
end;

procedure TSounderForm.OpenButtonClick(Sender: TObject);
begin
 StopButtonClick(nil);
 DataSize:=0;
 if OpenDialog1.Execute then
  OpenWaveFile;

 WavFileName:=OpenDialog1.FileName;
 Caption:='CyD Sounder - '+ExtractFileName(WavFileName);
{ if UseAcm then
  begin
   ChanelsBox.ItemIndex:=acmwfx.nChannels-1;
   FreqBox.Text:=IntToStr(acmwfx.nSamplesPerSec);
   BitsBox.Text:=IntToStr(acmwfx.wBitsPerSample);
  end
 else}
  begin
   if wfx.nChannels=1 then
    ChanelsBox.Text:='Mono'
   else
    ChanelsBox.Text:='Stereo';
   FreqBox.Text:=IntToStr(wfx.nSamplesPerSec);
   BitsBox.Text:=IntToStr(wfx.wBitsPerSample);
   Label7.Caption:=IntToStr(wfx.nAvgBytesPerSec);
  end;
end;

procedure TSounderForm.OpenWaveFile;
var
 hFile:THandle;
 Str:array [0..255] of char;
 TempStr:String;
 BytesReaded,Size:DWORD;
begin
 if w_hData<>0 then CloseHandle(w_hData);
 hFile:=CreateFile(PChar(OpenDialog1.FileName),GENERIC_READ,FILE_SHARE_READ,nil,OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE,0);
 wFileSize:=GetFileSize(hFile,nil);
 if hFile=INVALID_HANDLE_VALUE then exit;
 try
  try
//// Load Data
   SetFilePointer(hFile,8,nil,FILE_BEGIN);
   ReadFile(hFile,Str,8,BytesReaded,nil);
   Str[7]:=#0;
   TempStr:=Str;
   if TempStr<>'WAVEfmt' then
    begin
     Application.MessageBox('Wave file error','Error',MB_OK+MB_ICONINFORMATION);
     CloseHandle(hFile);
     exit;
    end;
   ReadFile(hFile,Size,Sizeof(Size),BytesReaded,nil);
   ZeroMemory(@wfx,sizeof(wfx));
   ZeroMemory(@acmwfx,sizeof(acmwfx));

   if Size=sizeof(PCMWAVEFORMAT) then
    begin
     ReadFile(hFile,wfx,Size,BytesReaded,nil);
//     USEACM:=false;
    end
   else
    begin
{     ReadFile(hFile,acmwfx,Size,BytesReaded,nil);
     wfx.wFormatTag:=WAVE_FORMAT_PCM;
     if acmFormatSuggest(0, acmwfx, wfx, sizeof(wfx),ACM_FORMATSUGGESTF_WFORMATTAG)<>MMSYSERR_NOERROR then
      begin
       MessageBox(Handle, 'Format not supported', 'Error', MB_OK+MB_ICONEXCLAMATION);
       exit;
      end;
     USEACM:=true;}
     MessageBox(Handle, 'Format not supported', 'Error', MB_OK+MB_ICONEXCLAMATION);
     exit;
    end;

   wfx.cbSize:=0;
   ReadFile(hFile,Str,4,BytesReaded,nil);
   ReadFile(hFile,DataSize,4,BytesReaded,nil);

   DataOffset:=SetFilePointer(hFile,0,nil,FILE_CURRENT);

   w_hData:=CreateFilemapping(hFile,nil,PAGE_READONLY,0,Integer(wFileSize),nil);
   CloseHandle(hFile);
   hFile:=INVALID_HANDLE_VALUE;
  except
   Application.MessageBox('The format of a file is not supported','Error',MB_OK+MB_ICONINFORMATION);
   CloseHandle(hFile);
   hFile:=INVALID_HANDLE_VALUE;
  end;
 finally
  if hFile<>INVALID_HANDLE_VALUE then CloseHandle(hFile);

  if DataSize>0 then
   begin
    EnableBut(true);
    InitSoundControls;
   end
  else
   EnableBut(false);
 end;
end;

procedure TSounderForm.PlayButtonClick(Sender: TObject);
begin
 if ((w_Pause)and (w_hPlay<>0)) then
  begin
   WaveOutCheck(waveOutRestart(w_hPlay));
   w_Pause:=not w_Pause;
   exit;
  end;

 if w_hPlay<>0 then StopButtonClick(nil);
 SoundTimer.Enabled:=true;
 w_Pause:=false; w_Restart:=false; w_Stop:=false;
 w_PlayPosition:=SounderForm.PositionBar.Position*1000;
 w_RestartPosition:=SounderForm.PositionBar.Position*1000;
 w_DeviceID:=SoundOptionForm.AudioDeviceCBox.ItemIndex;
 if SoundOptionForm.AudioDeviceCBox.ItemIndex=SoundOptionForm.AudioDeviceCBox.Items.Count-1 then
  w_DeviceID:=Integer(WAVE_MAPPER);
 CyDSounder1:=TCyDSounder.Create(true);
 CyDSounder1.Priority:=tpHigher;
 CyDSounder1.Resume;
end;

procedure TSounderForm.StopButtonClick(Sender: TObject);
begin
 if w_hPlay<>0 then
  begin
   w_Stop:=true;
   WaveOutCheck(waveOutReset(w_hPlay));
   SoundTimer.Enabled:=false;
  end;
 if SoundRecorder1<>nil then
  begin
   SoundRecorder1.Suspend;
   w_Stop:=true;
   waveInReset(SoundRecorder1.hRecord);
   SoundRecorder1.Resume;
  end;
end;

procedure TSounderForm.PouseButtonClick(Sender: TObject);
begin
 if w_hPlay=0 then exit;
 WaveOutCheck(waveOutPause(w_hPlay));
 w_Pause:=not w_Pause;
end;

procedure TSounderForm.SoundTimerTimer(Sender: TObject);
var
 mmt:MMTIME;
begin
 mmt.wType:=TIME_BYTES;
 waveOutCheck(waveOutGetPosition(w_hPlay, @mmt,sizeof(mmt)));
 PositionBar.Position:=round((mmt.cb+w_RestartPosition)/1000);
end;

procedure TSounderForm.EnableBut(Stat:Boolean);
begin
 PlayButton.Enabled:=Stat;
 StopButton.Enabled:=Stat;
 PouseButton.Enabled:=Stat;
 ConvButton.Enabled:=Stat;
 RecButton.Enabled:=Stat;
 PositionBar.Max:=round(DataSize/1000);
end;

procedure TSounderForm.FreqBarChange(Sender: TObject);
begin
 if w_hPlay=0 then exit;
 waveOutSetPlayBackRate(w_hPlay, round(FreqBar.Position/100));
end;

procedure TSounderForm.InitSoundControls;
var
 Value:DWord;
begin
 StopButtonClick(nil);
 WaveOutCheck(waveOutOpen(@w_hPlay, WAVE_MAPPER, @wfx, 0, 0,CALLBACK_NULL));
 waveOutGetPlayBackRate(w_hPlay, @Value);
 FreqBar.Position:=Value;
 waveOutGetPitch(w_hPlay, @Value);
 ToneBar.Position:=Value;
 waveOutClose(w_hPlay);
 w_hPlay:=0;
end;

procedure TSounderForm.ToneBarChange(Sender: TObject);
begin
 if w_hPlay=0 then exit;
 waveOutSetPitch(w_hPlay, round(FreqBar.Position/100));
end;

procedure TSounderForm.VolumeLBarChange(Sender: TObject);
var
 LValue,RValue:DWord;
begin
 LValue:=$FFFF-VolumeLBar.Position;
 RValue:=$FFFF-VolumeRBar.Position;
 WaveOutCheck(waveOutSetVolume(0, (RValue SHL 16)+LValue));
end;

procedure TSounderForm.PositionBarMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if w_hPlay=0 then exit;
 RestartPlay:=true;
 PouseButtonClick(nil);
end;

procedure TSounderForm.RecButtonClick(Sender: TObject);
begin
 if ((w_hPlay<>0)or (SoundRecorder1<>nil)) then
  StopButtonClick(nil);

 w_Pause:=false;
 w_Restart:=false;
 w_Stop:=false;
 w_RestartPosition:=PositionBar.Position*1000;
 StopButton.Enabled:=true;

 w_DeviceID:=SoundOptionForm.AudioDeviceCBox.ItemIndex;
 if SoundOptionForm.AudioDeviceCBox.ItemIndex=SoundOptionForm.AudioDeviceCBox.Items.Count-1 then
  w_DeviceID:=Integer(WAVE_MAPPER);

 CloseHandle(w_hData);

 SoundRecorder1:=TSoundRecorder.Create(true);
 SoundRecorder1.Priority:=tpHigher;
 SoundRecorder1.Resume;
end;

procedure TSounderForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 WriteOption;
 StopButtonClick(nil);
end;

procedure TSounderForm.ReadOption;
var
 FIniFile: TRegIniFile;
begin
 FIniFile := TRegIniFile.Create('Software');
 FIniFile.OpenKey('CyD',true);
 SoundOptionForm.AudioDeviceCBox.ItemIndex:=FIniFile.ReadInteger('Sounder', 'AudioIn', SoundOptionForm.AudioDeviceCBox.Items.Count-1);
 SoundOptionForm.RecDeviceCBox.ItemIndex:=FIniFile.ReadInteger('Sounder', 'AudioOut', SoundOptionForm.RecDeviceCBox.Items.Count-1);
 FIniFile.Free;
end;

procedure TSounderForm.OptionButtonClick(Sender: TObject);
begin
 SoundOptionForm.ShowModal;
 if SoundOptionForm.ModalResult<>mrOk then ReadOption;
end;

procedure TSounderForm.WriteOption;
var
 FIniFile: TRegIniFile;
begin
 FIniFile := TRegIniFile.Create('Software');
 FIniFile.OpenKey('CyD',true);
 FIniFile.WriteInteger('Sounder', 'AudioIn', SoundOptionForm.AudioDeviceCBox.ItemIndex);
 FIniFile.WriteInteger('Sounder', 'AudioOut', SoundOptionForm.RecDeviceCBox.ItemIndex);
 FIniFile.Free;
end;

procedure TSounderForm.ExitButtonClick(Sender: TObject);
begin
 Close;
end;

procedure TSounderForm.PositionBarMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 CalculateRestart;
end;

procedure TSounderForm.ConvButtonClick(Sender: TObject);
var
 afc:TACMFORMATCHOOSE;
 MaxSize:Integer;
 ftx:TWAVEFORMATEX;
begin
 CloseHandle(w_hData);
 ZeroMemory(@afc,sizeof(afc));
 afc.cbStruct:=sizeof(afc);
 MaxSize:=sizeof(TWAVEFORMATEX);
 acmMetrics(0,ACM_METRIC_MAX_SIZE_FORMAT, MaxSize);
 afc.pwfx:=@ftx;
 try
  afc.hwndOwner:=handle;
  afc.cbwfx:=MaxSize;
  afc.fdwEnum:=ACM_FORMATENUMF_CONVERT;
  afc.pwfxEnum:=@wfx;
  afc.pszTitle:='Select new format';
  if acmFormatChooseA(afc)=MMSYSERR_NOERROR then
   begin
    ConvertToFormat(@wfx,afc.pwfx);
    WavSaveToFile;
   end;
 except
 end;
end;

procedure TSounderForm.ConvertToFormat(oldFormat, newFormat: PWAVEFORMATEX);
var
 hStream:HACMSTREAM;
 Header:TACMSTREAMHEADER;
 hSRCFile, hDstFile:THandle;
 SourceSize, DestSize, Flags, BytesReaded:DWORD;
 SourceOffset, DestOffset:Integer;
begin
 if acmStreamOpen(@hStream,0,oldFormat^, newFormat^,nil,0,0,ACM_STREAMOPENF_NONREALTIME)<>MMSYSERR_NOERROR then
  begin
   MessageBox(Handle, 'Can''t convert data', 'Error', MB_OK+MB_ICONEXCLAMATION);
   exit;
  end;
 TempWavFile:=ExtractFilePath(Application.ExeName)+'Tempwave.dat';

 ZeroMemory(@Header,sizeof(Header));
 hSRCFile:=INVALID_HANDLE_VALUE;
 hDstFile:=INVALID_HANDLE_VALUE;
 DestOffset:=0;
 try
  ProgressBar1.Position:=0;
  Header.cbStruct:=sizeof(Header);
  SourceSize:=64*1024;
  DestSize:=SourceSize;
  acmStreamSize(hStream, SourceSize, DestSize, ACM_STREAMSIZEF_SOURCE);
  GetMem(Header.pbSrc,SourceSize);
  GetMem(Header.pbDst, DestSize);
  Header.cbSrcLength:=SourceSize;
  Header.cbDstLength:=DestSize;
  if acmStreamPrepareHeader(hStream, Header, 0)<>MMSYSERR_NOERROR then
   MessageBox(Handle, 'Can''t prepare stream header', 'Error', MB_OK+MB_ICONEXCLAMATION);
  hSrcFile:=CreateFile(PChar(WavFileName), GENERIC_READ, FILE_SHARE_READ, nil,  OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, 0);
  hDstFile:=CreateFile(PChar(TempWavFile), GENERIC_WRITE, 0, nil,  CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, 0);
  InitFile(hDstFile, newFormat);
  SetFilePointer(hSrcFile, DataOffset, nil, FILE_BEGIN);
  SourceOffset:=0;
  while SourceOffset<DataSize do
   begin
    Flags:=ACM_STREAMCONVERTF_BLOCKALIGN;
    if SourceOffset=0 then Flags:=Flags+ACM_STREAMCONVERTF_START;
    if Header.cbSrcLength>DataSize-SourceOffset then
     begin
      Header.cbSrcLength:=DataSize-SourceOffset;
      Flags:=Flags+ACM_STREAMCONVERTF_END;
     end;
    ReadFile(hSrcFile, Header.pbSrc^, Header.cbSrcLength, BytesReaded, nil);
    if acmStreamConvert(hStream, Header, Flags)<>MMSYSERR_NOERROR then
     MessageBox(Handle, 'Can''t convert data', 'Error', MB_OK+MB_ICONEXCLAMATION);

    WriteFile(hDstFile, Header.pbDst^, Header.cbDstLengthUsed, BytesReaded, nil);
    SourceOffset:=SourceOffset+Integer(Header.cbSrcLengthUsed);
    DestOffset:=DestOffset+Integer(Header.cbDstLengthUsed);
    if SourceOffset>DataSize then break;
    if Header.cbSrcLengthUsed<=0 then break;
    if Header.cbSrcLengthUsed<Header.cbSrcLength then
     SetFilePointer(hSrcFile, Integer(Header.cbSrcLengthUsed)+SourceOffset+DataOffset, nil, FILE_BEGIN);
    ProgressBar1.Position:=round(SourceOffset/DataSize*100);
   end;
 except
  on E: Exception do
   begin
    MessageBox(Handle, PChar(E.Message), 'Error', MB_OK+MB_ICONEXCLAMATION);
    if hSRCFile<>INVALID_HANDLE_VALUE then CloseHandle(hSrcFile);
    if hDstFile<>INVALID_HANDLE_VALUE then CloseHandle(hDstFile);
    ProgressBar1.Position:=0;
   end;
 end;
 UpdateWavFile(hDstFile, newFormat, DestOffset);

 acmStreamUnprepareHeader(hStream, Header, 0);
 acmStreamClose(hStream,0);
 if hSRCFile<>INVALID_HANDLE_VALUE then CloseHandle(hSrcFile);
 if hDstFile<>INVALID_HANDLE_VALUE then
  begin
   CloseHandle(hDstFile);
  end;
 ProgressBar1.Position:=0;

 FreeMem(Header.pbSrc);
 FreeMem(Header.pbDst);
end;

procedure TSounderForm.InitFile(hFile: THandle; wfx1:PWAVEFORMATEX);
var
 BytesRecorded,Data:DWord;
begin
 SetFilePointer(hFile,0,nil,FILE_BEGIN);
 WriteFile(hFile,'RIFF',4, BytesRecorded,nil);
 Data:=0;
 WriteFile(hFile,Data, 4,BytesRecorded,nil);
 WriteFile(hFile,'WAVEfmt ',8,BytesRecorded,nil);

 if wfx1.wFormatTag=WAVE_FORMAT_PCM then Data:=sizeof(PCMWAVEFORMAT)
 else Data:=sizeof(TWAVEFORMATEX)+wfx1.cbSize;

 WriteFile(hFile,Data,4,BytesRecorded,nil);
 WriteFile(hFile,wfx1^,Data,BytesRecorded,nil);
 WriteFile(hFile,'data',4,BytesRecorded,nil);
 Data:=0;
 WriteFile(hFile,Data,4,BytesRecorded,nil);
end;

procedure TSounderForm.WavSaveToFile;
begin
 if w_hData<>0 then CloseHandle(w_hData);
 MoveFileEx(PChar(TempWavFile), PChar(WavFileName), MOVEFILE_REPLACE_EXISTING);
 OpenWaveFile;
end;

procedure TSounderForm.UpdateWavFile(hFile: THandle; wfx1: PWAVEFORMATEX;
  newDataSize: Integer);
var
 BytesRecorded, Data, FmtSize:dWord;
begin
 if wfx1.wFormatTag=WAVE_FORMAT_PCM then FmtSize:=sizeof(PCMWAVEFORMAT)
 else FmtSize:=sizeof(TWAVEFORMATEX)+wfx1.cbSize;
 Data:=newDataSize+Integer(fmtSize)+12;
 SetFilePointer(hFile,4,nil,FILE_BEGIN);
 WriteFile(hFile,Data,4, BytesRecorded,nil);

 SetFilePointer(hFile,24+FmtSize,nil,FILE_BEGIN);
 WriteFile(hFile,newDataSize,4, BytesRecorded,nil);
end;

procedure TSounderForm.NewButtonClick(Sender: TObject);
var
 hFile:THandle;
begin
 EnableBut(false);
 NewFileForm.ShowModal;

 if NewFileForm.ModalResult<>mrOk then exit;
 if NewFileForm.WavFilenameEdit.FileName='' then
  begin
   MessageBox(SounderForm.Handle,'Wrong name of a file', 'Error', MB_OK+MB_ICONEXCLAMATION);
   exit;
  end;

 Caption:='CyD Sounder - '+NewFileForm.WavFilenameEdit.FileName;
 WavFileName:=NewFileForm.WavFilenameEdit.FileName;

 PositionBar.Max:=1;
 wFileSize:=0;

 wfx.nChannels:=NewFileForm.ChanelsBox.ItemIndex+1;
 wfx.nSamplesPerSec:=StrToInt(NewFileForm.FreqBox.Text);
 wfx.wBitsPerSample:=StrToInt(NewFileForm.BitsBox.Text);
 wfx.wFormatTag:=WAVE_FORMAT_PCM;
 wfx.nBlockAlign:=round(wfx.wBitsPerSample/8*wfx.nChannels);
 wfx.nAvgBytesPerSec:=wfx.nSamplesPerSec*wfx.nBlockAlign;
 wfx.cbSize:=0;
 hFile:=CreateFile(PChar(SounderForm.WavFileName),GENERIC_WRITE,0,nil,CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE,0);
 DataOffset:=InitNewFile(hFile);
 DataSize:=0;
 CloseHandle(hFile);
 EnableBut(true);
 if NewFileForm.ChanelsBox.ItemIndex=0 then
  ChanelsBox.Text:='Mono'
 else
  ChanelsBox.Text:='Stereo';
 FreqBox.Text:=NewFileForm.FreqBox.Text;
 BitsBox.Text:=NewFileForm.BitsBox.Text;
end;

procedure TSounderForm.SaveRecFile;
begin
 OpenDialog1.FileName:=WavFileName;
 OpenWaveFile;
 exit;
end;

function TSounderForm.InitNewFile(hFile: THandle): Integer;
var
 BytesRecorded,Data:DWord;
begin
 SetFilePointer(hFile,0,nil,FILE_BEGIN);
 WriteFile(hFile,'RIFF',4, BytesRecorded,nil);
 Data:=0;
 WriteFile(hFile,Data, 4,BytesRecorded,nil);
 WriteFile(hFile,'WAVEfmt ',8,BytesRecorded,nil);
 Data:=sizeof(PCMWAVEFORMAT);
 WriteFile(hFile,Data,4,BytesRecorded,nil);
 WriteFile(hFile,wfx,Sizeof(PCMWAVEFORMAT),BytesRecorded,nil);
 WriteFile(hFile,'data',4,BytesRecorded,nil);
 Data:=0;
 WriteFile(hFile,Data,4,BytesRecorded,nil);
 result:=28+sizeof(PCMWAVEFORMAT);
end;


procedure TSounderForm.CalculateRestart;
begin
 if PositionBar.Position<DataSize then
  begin
   w_Restart:=true;
   w_RestartPosition:=PositionBar.Position;
   w_RestartPosition:=w_RestartPosition-(w_RestartPosition mod wfx.nBlockAlign);
  end
 else
  StopButtonClick(nil);
 if RestartPlay=true then
  PouseButtonClick(nil);
 RestartPlay:=false;  
end;

end.
